@@ -6,9 +6,14 @@ from AccessControl.SecurityManagement import newSecurityManager
 from AccessControl.SecurityManagement import noSecurityManager
 from Testing import makerequest
 from optparse import OptionParser
+try:
+    from Products.PloneTestCase import version
+except ImportError:
+    version = None
 pre_plone3 = False
 try:
-    from plone.app.linkintegrity.exceptions import LinkIntegrityNotificationException
+    from plone.app.linkintegrity.exceptions import \
+        LinkIntegrityNotificationException
 except ImportError:
     # we are using a release prior to 3.x
     pre_plone3 = True
@@ -58,8 +63,14 @@ def create(app, site_id, products_initial, profiles_initial, site_replace):
             return
     # actually add in Plone
     if site_id not in oids:
-        factory = app.manage_addProduct['CMFPlone']
-        factory.addPloneSite(site_id, create_userfolder=1)
+        if version is not None and version.PLONE40:
+            from Products.CMFPlone.factory import addPloneSite
+            addPloneSite(app, site_id)
+        else:
+            factory = app.manage_addProduct['CMFPlone']
+            factory.addPloneSite(site_id, create_userfolder=1)
+        # commit the new site to the database
+        transaction.commit()
         print "Added Plone Site"
     # install some products
     plone = getattr(app, site_id)
@@ -76,7 +87,7 @@ def main(app, parser):
     admin_user = options.admin_user
     post_extras = options.post_extras
     pre_extras = options.pre_extras
-    
+
     # normalize our product/profile lists
     products_initial = getProductsWithSpace(options.products_initial)
     products = getProductsWithSpace(options.products)
@@ -96,25 +107,25 @@ def main(app, parser):
     # create the plone site if it doesn't exist
     create(app, site_id, products_initial, profiles_initial, site_replace)
     portal = getattr(app, site_id)
-    
+
     def runExtras(portal, script_path):
         if os.path.exists(script_path):
             execfile(script_path)
         else:
             msg = 'The path to the extras script does not exist: %s'
             raise zc.buildout.UserError(msg % script_path)
-    
+
     for pre_extra in pre_extras:
         runExtras(portal, pre_extra)
-    
+
     if products:
         quickinstall(portal, products)
     if profiles:
         runProfiles(portal, profiles)
-    
+
     for post_extra in post_extras:
         runExtras(portal, post_extra)
-    
+
     # commit the transaction
     transaction.commit()
     noSecurityManager()
